import math
import imp
import time
imp.load_source("util","ScriptEditor/util.py") 
from util import *
imp.load_source("lgFactors","ScriptEditor/lgFactors.py") 
from lgFactors import *
imp.load_source("l2dResults","ScriptEditor/l2dResults.py") 
from l2dResults import *
imp.load_source("boundary","ScriptEditor/boundary.py") 
from boundary import *
imp.load_source("boundary","ScriptEditor/simulationResults.py") 
from boundary import *


currentGeometry = 0
currentFill = 0
currentBoundary = 0

startTime = time.time()

if "default_geometry_id" in Input:
	currentGeometry = Input["default_geometry_id"]

simulationSettings = Simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)

Output = dict()
OutputDescriptors = list()

geometryInfo = Simulator.getGeometryInfo()
specialElementsInfo = Simulator.getSpecialElementsInfo(currentGeometry)
fillInfo = Simulator.getFillInfo(currentGeometry)
boundaryInfo = Simulator.getBoundaryInfo(currentGeometry, currentFill)
boundaryConditions = Simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)

specialElementsConfig = list()
if specialElementsInfo[0]["element"]["__descriptor_name__"] == "__bridge_section_elements__":
	# Just tell simulator to get a description of section elements
	specialElementsConfig.append(1)
elif specialElementsInfo[0]["element"]["__descriptor_name__"] == "__10077-2_insulation_panel__":
	# Just tell simulator to get a description of section elements
	specialElementsConfig.append(3)
Input["boundary_variant"] = "flows"
flowSimulation, flowSimulationDescriptors = addSimulation(Simulator,\
														currentGeometry,\
														currentFill,\
														currentBoundary,\
														specialElementsConfig,\
														boundaryConditions,\
														Input,\
														Output,\
														OutputDescriptors,\
														_("Flow simulation"),\
														"flow_s")

simulation_3d = "3d_simulation" in flowSimulation and flowSimulation["3d_simulation"]
bridgeSectionElements = specialElementsInfo[0]["element"]["__descriptor_name__"] == "__bridge_section_elements__" or simulation_3d
insulationPanel10077 = specialElementsInfo[0]["element"]["__descriptor_name__"] == "__10077-2_insulation_panel__" and not simulation_3d
														
computeDew = False
computeUfPsi = True
computeUtjPsij = False	
if "flows" in simulationSettings:
	if "compute_dew" in simulationSettings["flows"]:
		computeDew = simulationSettings["flows"]["compute_dew"]
if "simulation_settings" in simulationSettings:		
	if "computer_uf_psi" in simulationSettings["simulation_settings"]:
		computeUfPsi = simulationSettings["simulation_settings"]["computer_uf_psi"]
	if "computer_utj_psij" in simulationSettings["simulation_settings"]:
		computeUtjPsij = simulationSettings["simulation_settings"]["computer_utj_psij"]
		
if computeDew:
	Input["boundary_variant"] = "dew"
	specialElementsConfig = list()
	if bridgeSectionElements:
		# Tell simulator to perform analysis on section elements
		specialElementsConfig.append(2)
	elif insulationPanel10077:
		specialElementsConfig.append(0)
	dewSimulation, dewSimulationDescriptors = addSimulation(Simulator,\
															currentGeometry,\
															currentFill,\
															currentBoundary,\
															specialElementsConfig,\
															boundaryConditions,\
															Input,\
															Output,\
															OutputDescriptors,\
															_("Surf. Cond."),\
															"dew_s")

computeMold = False
if "flows" in simulationSettings:
	if "compute_mold" in simulationSettings["flows"]:
		computeMold = simulationSettings["flows"]["compute_mold"]
if computeMold:
	Input["boundary_variant"] = "mold"
	specialElementsConfig = list()
	if bridgeSectionElements:
		# Tell simulator to perform analysis on section elements
		specialElementsConfig.append(2)
	elif insulationPanel10077:
		specialElementsConfig.append(0)

	condensationSimulation, condensationSimulationDescriptors = addSimulation(Simulator,\
																			currentGeometry,\
																			currentFill,\
																			currentBoundary,\
																			specialElementsConfig,\
																			boundaryConditions,\
																			Input,\
																			Output,\
																			OutputDescriptors,\
																			_("Mold"),\
																			"cond_s")															

panelSimulation10077 = object()

performPanelSimulation = insulationPanel10077 and specialElementsInfo[0]["num_cycles"] > 0 and computeUfPsi
if performPanelSimulation:
	# Perform a simulation substituting delimited areas with an insulation panel
	specialElementsConfig = list()
	specialElementsConfig.append(1)
	Input["boundary_variant"] = "flows"
	panelSimulation10077, panelSimulation10077Descriptors = addSimulation(Simulator,\
																	currentGeometry,\
																	currentFill,\
																	currentBoundary,\
																	specialElementsConfig,\
																	boundaryConditions,\
																	Input,\
																	Output,\
																	OutputDescriptors,\
																	_("10077 Insulation panel"),\
																	"10077_ip_s")
	#panelSimulation10077Descriptors.insert(0, createNumberDescriptor("heat_flow", _("Flow"), "W/m", "%.4f"))
	#panelSimulation10077["heat_flow"] = getAvFlow(panelSimulation10077["advanced_results"]["boundary"]["tf_analysis"])
	#flowSimulationDescriptors.insert(0, createNumberDescriptor("heat_flow", _("Flow"), "W/m", "%.4f"))
	#flowSimulation["heat_flow"] = getAvFlow(flowSimulation["advanced_results"]["boundary"]["tf_analysis"])
else:
	panelSimulation10077 = flowSimulation
	panelSimulation10077Descriptors = list()
	#panelSimulation10077["heat_flow"] = getAvFlow(panelSimulation10077["advanced_results"]["boundary"]["tf_analysis"])

advancedResults = flowSimulation["advanced_results"]
advancedResultsDescriptors = findSubDescriptors("advanced_results", flowSimulationDescriptors)

boundaryResults = advancedResults["boundary"]
boundaryResultsDescriptors = findSubDescriptors("boundary",  advancedResultsDescriptors)

# Get temperatures to be used for L-G computation
boundaryGroups = boundaryResults["groups"]
lgAirTemperature1, lgAirTemperature2 = getLGAirTemperatures(boundaryConditions, boundaryGroups)

# Generare groups name
boundaryGroupNames = generateBoundaryGroupsNames(boundaryGroups, boundaryConditions)

deltaT = computeDeltaT(boundaryResults, boundaryGroups)

# Compute L and G values
computeL = False
computeG = False
if "flows" in simulationSettings:
	computeL = simulationSettings["flows"]["compute_l"]
	computeG = simulationSettings["flows"]["compute_g"] and computeMold
	
if computeL:
	boundaryResultsDescriptors.append(createLResultsTableDescriptor(boundaryGroups, simulation_3d))

	# L results
	lComputationsGrid = generateLResultsGrid(len(boundaryGroups))
	boundaryResults["l_computations"] = computeLResultsTable(Simulator,\
															currentGeometry,\
															currentFill,\
															currentBoundary,\
															flowSimulation,\
															Input,\
															Output,\
															OutputDescriptors,\
															lgAirTemperature1,\
															lgAirTemperature2,\
															lComputationsGrid,\
															boundaryGroups)

	# Create L  table
	boundaryResultsDescriptors.append(createLTableDescriptor(boundaryGroupNames, simulation_3d))
	boundaryResults["l_table"] = computeLTable(boundaryGroupNames, boundaryResults["l_computations"])

	# Create flow table
	boundaryResultsDescriptors.append(createFlowTableDescriptor(boundaryGroupNames, simulation_3d))
	boundaryResults["flow_table"] = computeFlowsTable(boundaryResults, boundaryConditions, boundaryGroups)

	# L2D computations
	bridgedL2D, psi, additionalPsi = computeBridgedL2D(boundaryResults["l_computations"],\
														boundaryResults["l_table"],\
														boundaryResults["flow_table"],\
														boundaryGroups,\
														advancedResults)
else:
	bridgedL2D = computeBridgedL2DNoL(boundaryResults, deltaT)
	psi = [0.0]
	additionalPsi = list()
	
if computeG:
	condensationSimulationAdvanced = condensationSimulation["advanced_results"]
	condensationSimulationBoundary = condensationSimulationAdvanced["boundary"]
	condensationSimulationAdvancedDescriptors = findSubDescriptors("advanced_results",  condensationSimulationDescriptors)
	condensationSimulationBoundaryDescriptors = findSubDescriptors("boundary",  condensationSimulationAdvancedDescriptors)
	
	condensationSimulationBoundaryDescriptors.append(createGTableDescriptor(boundaryGroupNames))
	condensationSimulationBoundary["g_table"] = computeGTable(Simulator,\
																currentGeometry,\
																currentFill,\
																currentBoundary,\
																condensationSimulation,\
																Input,\
																Output,\
																OutputDescriptors,\
																boundaryGroups,\
																boundaryGroupNames)
						
if insulationPanel10077:	
	noBridgeL2D, additionalResults = computeNoBridgeL2D10077(boundaryGroups,\
															advancedResults,\
															panelSimulation10077,\
															bridgedL2D,\
															deltaT,\
															computeUfPsi,\
															computeUtjPsij)
	
	biraSettings = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "boundary_increased_resistance")															
	add10077BoundaryResults(boundaryResults,\
							boundaryGroups,\
							biraSettings,\
							Output,\
							OutputDescriptors)
	
	add10077UfResults(bridgedL2D,\
						additionalResults,\
						Output,\
						OutputDescriptors,\
						flowSimulation,\
						flowSimulationDescriptors,\
						panelSimulation10077,\
						panelSimulation10077Descriptors)
else:
	noBridgeL2D = computeNoBridgeL2D(deltaT,\
										advancedResults,\
										Simulator,\
										currentGeometry,\
										currentFill,\
										currentBoundary,\
										Input,\
										Output,\
										OutputDescriptors)
	if simulation_3d:
		noBridgeL1D_3D = computeNoBridgeL1D_3D(advancedResults,\
											Simulator,\
											currentGeometry)
		noBridgeL2D_3D = computeNoBridgeL2D_3D(advancedResults,\
											Simulator,\
											currentGeometry)
		addL1DResults_3D(noBridgeL1D_3D,
						flowSimulation,\
						flowSimulationDescriptors)
		addL2DResults_3D(noBridgeL2D_3D,
						flowSimulation,\
						flowSimulationDescriptors)
		addL3DResults_3D(bridgedL2D,\
						flowSimulation,\
						flowSimulationDescriptors)
		addPSIResults_3D(bridgedL2D,\
						noBridgeL1D_3D,\
						noBridgeL2D_3D,\
						flowSimulation,\
						flowSimulationDescriptors)			

if not simulation_3d:						
	addUResults(bridgedL2D,\
				flowSimulation,\
				flowSimulation,\
				flowSimulationDescriptors)
	if not insulationPanel10077:
		addL2DResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						additionalPsi,\
						flowSimulation,\
						flowSimulation,\
						flowSimulationDescriptors,\
						insulationPanel10077,\
						computeUfPsi)
		addFlowResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						deltaT,\
						flowSimulation,\
						flowSimulationDescriptors)
	else:
		addL2DResults(bridgedL2D,\
						noBridgeL2D,\
						psi,\
						additionalPsi,\
						flowSimulation,\
						Output,\
						OutputDescriptors,\
						insulationPanel10077,\
						computeUfPsi)

	if("l2d_bridged" in Output and bridgedL2D == 0.0):
		Output["error"] = _("Bridged L2D is zero. Please check your project")
		
addSimulationTimeResult(startTime, OutputDescriptors, Output)

pslgBoundarySettings = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "pslg_boundary")
pslgBoundaryDescriptors = Simulator.getSettings(currentGeometry, currentFill, currentBoundary, "pslg_boundary_descriptors")
addCavityDetectionResult(pslgBoundarySettings, \
						pslgBoundaryDescriptors, \
						OutputDescriptors, \
						Output)
