import math
import gettext
import locale
import csv
import time
import imp
imp.load_source("boundaryUtil","ScriptEditor/boundaryUtil.py")  
from boundaryUtil import *

# Get string translation for a specific language
if locale.getlocale()[0] == "Italian_Italy" or locale.getlocale()[0] == "it_IT":
	translator = gettext.GNUTranslations(open("it_IT/heatSimulator.mo", "rb"))
elif locale.getlocale()[0] == "Spanish_spain" or locale.getlocale()[0] == "es_ES":
	translator = gettext.GNUTranslations(open("es_ES/heatSimulator.mo", "rb"))
else:
	translator = gettext.NullTranslations()

# Search for a descriptor by its bind name and return its index inside a vector of descriptors
def findDescriptor(bind, descriptors):
	count = int(0)
	for descriptor in descriptors:
		if descriptor["bind"] == bind:
			return count
		else:
			count = count + 1
	return -1

# Get descriptors of a parameter map parameter child of another parameter
def findSubDescriptors(bind, descriptors):
	return descriptors[findDescriptor(bind, descriptors)]["parameter_map"]

# Create a number descriptor
def createNumberDescriptor(bind,\
							name, \
							unit, \
							format, \
							tag = "", \
							visible = True, \
							callbackButton = "", \
							centerHorizontal = False, \
							size = list(), \
							labelSize = list(), \
							span = list()):
	descriptor = dict()
	descriptor["type"] = "double"
	descriptor["bind"] = bind
	descriptor["name"] = name
	descriptor["unit"] = unit
	descriptor["format"] = format
	descriptor["default"] = 0.0
	if tag != "":
		descriptor["__tag__"] = tag
	descriptor["visible"] = visible
	if callbackButton != "":
		descriptor["callback_button"] = callbackButton
	descriptor["center_horizontal"] = centerHorizontal
	if size:
		descriptor["size"] = size
	if labelSize:
		descriptor["label_size"] = labelSize
	if span:
		descriptor["span"] = span
	return descriptor

# Create a string descriptor
def createStringDescriptor(bind, name, tag = "", width = -1, height = -1, visible = True):
	descriptor = dict()
	descriptor["type"] = "string"
	descriptor["bind"] = bind
	descriptor["name"] = name
	if tag != "":
		descriptor["__tag__"] = tag
	descriptor["default"] = "-"
	if width != -1 or height != -1:
		descriptor["size"] = list()
		descriptor["size"].append(int(width))
		descriptor["size"].append(int(height))
	descriptor["visible"] = visible
	return descriptor	

# Create a grid descriptor
def createGridDescriptor(bind, name, colDescriptors, width = -1, height = -1):
	descriptor = dict()
	descriptor["type"] = "grid"
	descriptor["bind"] = bind
	descriptor["name"] = name
	if width != -1 or height != -1:
		descriptor["size"] = list()
		descriptor["size"].append(int(width))
		descriptor["size"].append(int(height))
	descriptor["grid_cols"] = colDescriptors
	return descriptor

# Create an empty grid descriptor
def createEmptyGridDescriptor(bind, name, tag = "", width = -1, height = -1):
	descriptor = dict()
	descriptor["type"] = "grid"
	descriptor["bind"] = bind
	descriptor["name"] = name
	if width != -1 or height != -1:
		descriptor["size"] = list()
		descriptor["size"].append(int(width))
		descriptor["size"].append(int(height))
	if tag != "":
		descriptor["__tag__"] = tag
	descriptor["grid_cols"] = list()
	return descriptor

# Add a number column descriptor to a grid
def addGridColNumberDescriptor(gridDescriptor, bind, name, unit, format):
	gridDescriptor["grid_cols"].append(createNumberDescriptor(bind, name, unit, format))
	return gridDescriptor

# Add a string column descriptor to a grid	
def addGridColStringDescriptor(gridDescriptor, bind, name):
	gridDescriptor["grid_cols"].append(createStringDescriptor(bind, name))
	return gridDescriptor
	
def createSeparatorDescriptor(bind, span = list()):
	descriptor = dict()
	descriptor["type"] = "separator"
	descriptor["bind"] = bind
	if span:
		descriptor["span"] = span
	return descriptor

# Add a simulation to output	
def addSimulation(simulator,\
					currentGeometry,\
					currentFill,\
					currentBoundary,\
					specialElementsConfig,\
					boundaryConditions,\
					input,\
					output,\
					outputDescriptors,\
					simulationName,\
					simulationTag = ""):

	startTime = time.time()
					
	simulationOutput, simulationOutputDescriptors = simulator.simulate(currentGeometry,\
																		currentFill,\
																		currentBoundary,\
																		specialElementsConfig,\
																		boundaryConditions,\
																		input)

	currentSimulation = 1
	while "simulation_" + str(currentSimulation) in output:
		currentSimulation = currentSimulation + 1

	simulationTimeDescriptor = createNumberDescriptor("sim_time", _("Simulation time"), "s", "%.2f", "sim_time", False)
	simulationOutputDescriptors.append(simulationTimeDescriptor)
	simulationOutput["sim_time"] = time.time() - startTime
	
	simulationBind = "simulation_" + str(currentSimulation)
	
	simulationParameter = dict()
	simulationParameter["name"] = simulationName
	simulationParameter["bind"] = simulationBind
	simulationParameter["__descriptor_name__"] = "__simulation" + str(currentSimulation) + "__"
	simulationParameter["type"] = "parameter_map"
	simulationParameter["parameter_map_style"] = "tabbed"
	simulationParameter["parameter_map"] = simulationOutputDescriptors
	# Save also the next tag fow backward compatibility
	simulationParameter["__tag___old"] = "sim" + str(currentSimulation)
	if simulationTag == "":
		simulationParameter["__tag__"] = simulationParameter["__tag___old"]
	else:
		simulationParameter["__tag__"] = simulationTag
		
	outputDescriptors.append(simulationParameter)
	output[simulationBind] = simulationOutput
	return simulationOutput, simulationOutputDescriptors

def boundaryInGroup(boundaryID, boundaryGroup):
	for boundaryIDGroup in boundaryGroup:
		if boundaryIDGroup == boundaryID:
			return True
	return False
	
def getAvFlow(tfAnalysis):
	positiveQ = 0.0
	nagativeQ = 0.0
	for boundaryID in range(len(tfAnalysis)):
		boundaryQ = tfAnalysis[boundaryID]["Q"]
		if boundaryQ > 0.0:
			positiveQ = positiveQ + boundaryQ
		else:
			nagativeQ = nagativeQ + boundaryQ
	return (positiveQ + math.fabs(nagativeQ)) * 0.5
	
def getAvFlow3D(tfAnalysis, boundaryConditions):
	positiveQ = 0.0
	nagativeQ = 0.0
	adiabaticQ = 0.0
	for boundaryID in range(len(tfAnalysis)):
		boundaryQ = tfAnalysis[boundaryID]["Q"]
		if boundaryConditions[boundaryID]["type"] == "adiabatic":
			adiabaticQ = adiabaticQ + boundaryQ
			continue
		if boundaryQ > 0.0:
			positiveQ = positiveQ + boundaryQ
		else:
			nagativeQ = nagativeQ + boundaryQ
	#return (positiveQ + math.fabs(nagativeQ)) * 0.5
			
	if positiveQ > math.fabs(nagativeQ):
		return positiveQ + math.fabs(adiabaticQ)# * 2.0
	else:
		return math.fabs(nagativeQ) + math.fabs(adiabaticQ)# * 2.0
	
def computeFlow(boundaryResults):
	flow = 0.0
	boundaryGroups = boundaryResults["groups"]
	
	if len(boundaryGroups) == 2:
		flow1 = 0.0
		flow2 = 0.0
		for boundaryID in boundaryGroups[0]:
			if boundaryID < len(boundaryResults["tf_analysis"]):
				flow1 = flow1 + boundaryResults["tf_analysis"][boundaryID]["Q"]
		for boundaryID in boundaryGroups[1]:
			if boundaryID < len(boundaryResults["tf_analysis"]):
				flow2 = flow2 + boundaryResults["tf_analysis"][boundaryID]["Q"]
		flow = (math.fabs(flow1) + math.fabs(flow2)) * 0.5
	else:
		flow1 = 0.0
		flow2 = 0.0

		couplingsIDs = boundaryResults["monthly_couplings_ids"]
		processedIDs = dict()
		for couplingIDs in couplingsIDs:
			internalID = couplingIDs["internal"]
			externalID = couplingIDs["external"]
			if not internalID in processedIDs:
				flow1 = flow1 + boundaryResults["tf_analysis"][internalID]["Q"]
				processedIDs[internalID] = True
			if not externalID in processedIDs:
				flow2 = flow2 + boundaryResults["tf_analysis"][externalID]["Q"]
				processedIDs[externalID] = True
		flow = (math.fabs(flow1) + math.fabs(flow2)) * 0.5
					
	return flow	
	
def loadCSV(file, requiredCols, optionals = dict()):
	elements = list()
	csvreader = csv.reader(open(file))
	firstRow = True
	for row in csvreader:
		if firstRow:
			for requiredCol in requiredCols:
				if not requiredCol in row and not requiredCol in optionals:
					raise NameError("\"" + requiredCol + "\"" + " column not found in \"" + file + "\"")
			colNames = row
			firstRow = False
		else:
			element = dict()
			cellID = 0
			for key, value in optionals.items():
				if value != "":
					element[key] = value
			for cell in row:
				colName = colNames[cellID]
				if cell != "" or not colName in optionals.keys():
					element[colName] = cell
				cellID = cellID + 1
			if len(element) >= len(requiredCols) - len(optionals):
				elements.append(element)
	return elements
	
